pub mod md5 {
    use md5::Digest;
    pub fn hash<T: AsRef<[u8]>>(input: T) -> String {
        let mut hasher = md5::Md5::new();
        hasher.update(input);
        let result = hasher.finalize();
        crate::serialize::to_hex(result.as_slice())
    }
}

pub mod sha1 {
    use sha1::Digest;
    pub fn hash<T: AsRef<[u8]>>(input: T) -> String {
        let mut hasher = sha1::Sha1::new();
        hasher.update(input);
        let result = hasher.finalize();
        crate::serialize::to_hex(result.as_slice())
    }
}

pub mod sha512 {
    pub fn hash<T: AsRef<[u8]>>(input: T) -> String {
        let mut hasher = hmac_sha512::Hash::new();
        hasher.update(input);
        let result = hasher.finalize();
        crate::serialize::to_hex(result.as_slice())
    }
}

pub mod image_hash {
    pub enum Precision {
        Low = 2,
        Medium = 8,
        High = 18,
        Detailed = 32,
    }

    pub fn hash_image<T: image::GenericImage<Pixel = image::Rgb<u8>>>(
        input: &T,
        precision: Precision,
    ) -> String {
        match precision {
            Precision::Low => blockhash::blockhash16(input).to_string(),
            Precision::Medium => blockhash::blockhash64(input).to_string(),
            Precision::High => blockhash::blockhash144(input).to_string(),
            Precision::Detailed => blockhash::blockhash256(input).to_string(),
        }
    }
}
